@@ -3,20 +3,10 @@
 #'
 #' @description  The core analysis is done by ustat11 with the difference kernel (kernelFlag = 2).
 #'   This calculation can also be accomplished by ustat11 with the identity kernel (kernelFlag = 1),
-#'   and the code to do that is provided after the return statement so it never gets exectuted.
+#'   and the code to do that is provided after the return statement so it never gets executed.
 #'
-#' @param df
-#' Data frame of observations, one per row. Columns identify random effects, fixed effects,
-#' and the observation. Namely,
-#' \itemize{
-#'   \item \code{readerID:} The factor corresponding to the different readers in the study.
-#'     The readerID is treated as a random effect.
-#'   \item \code{caseID:} The factor corresponding to the different cases in the study.
-#'     The caseID is treated as a random effect.
-#'   \item \code{modalityID:} The factor corresponding to the different modalities in the study.
-#'     The modalityID is treated as a fixed effect.
-#'   \item \code{score:} The score given by the reader to the case for the modality indicated.
-#' }
+#' @param df an iMRMC formatted data frame, see \link{dfMRMC_example}
+#' 
 #'
 #' @param modalitiesToCompare
 #' The factors identifying the modalities to compare.
@@ -30,16 +20,16 @@
 #' @export
 #'
 laWRBM <- function(
-  df, modalitiesToCompare,
-  keyColumns = c("readerID", "caseID", "modalityID", "score")
+    df, modalitiesToCompare,
+    keyColumns = c("readerID", "caseID", "modalityID", "score")
 ) {
-
+  
   if (length(modalitiesToCompare) != 2) {
     print(paste("length(modalitiesToCompare) =", length(modalitiesToCompare)))
     stop("ERROR: modalitiesToCompare should have 2 elements.")
   }
   mToCompare <- c(modalitiesToCompare, modalitiesToCompare)
-
+  
   # The limits of agreement are determined by the MRMC moments of uStat11
   # when the kernel flag is 2.
   # The limits can also be determined from uStat11 when the kernel flag is 1,
@@ -48,46 +38,46 @@ laWRBM <- function(
     df, keyColumns = keyColumns,
     modalitiesToCompare = mToCompare, kernelFlag = 2
   )
-
+  
   meanDiff <- result1$mean[1]
   var.MeanDiff <- result1$var[1]
   var.1obs <- result1$var.1obs[1]
-
+  
   la.bot <- meanDiff - 2 * sqrt(var.1obs)
   la.top <- meanDiff + 2 * sqrt(var.1obs)
-
+  
   ci95meanDiff.bot <- meanDiff + qnorm(.025) * sqrt(var.MeanDiff)
   ci95meanDiff.top <- meanDiff + qnorm(.975) * sqrt(var.MeanDiff)
-
+  
   result2 <- data.frame(
     meanDiff = meanDiff, var.MeanDiff = var.MeanDiff, var.1obs = var.1obs,
     ci95meanDiff.bot = ci95meanDiff.bot, ci95meanDiff.top = ci95meanDiff.top,
     la.bot = la.bot, la.top = la.top, stringsAsFactors = TRUE)
-
+  
   return(result2)
-
+  
   #### Alternate method with identical result ####
   #### Uses uStat11 and the identity kernel
   result <- uStat11.jointD(
     df, keyColumns = keyColumns,
     modalitiesToCompare = modalitiesToCompare, kernelFlag = 1)
-
+  
   moments <- result$moments
-
+  
   var.Arc = moments$c1r1[1] - moments$c0r0[1]
   var.Brc = moments$c1r1[2] - moments$c0r0[2]
   cov.Ar1cBr1c <- moments$c1r1[3] - moments$c0r0[3]
-
+  
   var.Ar1cminusBr1c <- var.Arc + var.Brc - 2 * cov.Ar1cBr1c
-
+  
   print(result2$var)
   print(var.Ar1cminusBr1c)
-
+  
   print("You should not reach this point because of a previous return statement.")
   browser()
-
+  
   return(result2)
-
+  
 }
 
 #---- laBRBM ----
@@ -95,18 +85,8 @@ laWRBM <- function(
 #'
 #' @description The core analysis is done by ustat11 with the identity kernel (kernelFlag = 1).
 #'
-#' @param df
-#' Data frame of observations, one per row. Columns identify random effects, fixed effects,
-#' and the observation. Namely,
-#' \itemize{
-#'   \item \code{readerID:} The factor corresponding to the different readers in the study.
-#'     The readerID is treated as a random effect.
-#'   \item \code{caseID:} The factor corresponding to the different cases in the study.
-#'     The caseID is treated as a random effect.
-#'   \item \code{modalityID:} The factor corresponding to the different modalities in the study.
-#'     The modalityID is treated as a fixed effect.
-#'   \item \code{score:} The score given by the reader to the case for the modality indicated.
-#' }
+#' @param df an iMRMC formatted data frame, see \link{dfMRMC_example}
+#' 
 #'
 #' @param modalitiesToCompare
 #' The factors identifying the modalities to compare.
@@ -120,67 +100,67 @@ laWRBM <- function(
 #' @export
 #'
 laBRBM <- function(
-  df, modalitiesToCompare,
-  keyColumns = c("readerID", "caseID", "modalityID", "score")
+    df, modalitiesToCompare,
+    keyColumns = c("readerID", "caseID", "modalityID", "score")
 ) {
-
+  
   if (length(modalitiesToCompare) != 2) {
     print(paste("length(modalitiesToCompare) =", length(modalitiesToCompare)))
     stop("ERROR: modalitiesToCompare should have 2 elements.")
   }
-
+  
   # Estimate the BRBM limits of agreement
   # The limits of agreement are determined by the MRMC moments of uStat11
   # when the kernel flag is 1.
   result1 <- uStat11.conditionalD(
     df, modalitiesToCompare = modalitiesToCompare,
     kernelFlag = 1, keyColumns = keyColumns)
-
+  
   meanDiff <- result1$mean[3]
   var.MeanDiff <- result1$var[3]
-
+  
   moments <- result1$moments
-
+  
   var.Arc = result1$var.1obs[1]
   var.Brc = result1$var.1obs[2]
   cov.Ar1cBr2C <- moments$c1r0[3] - moments$c0r0[3]
-
+  
   var.Ar1cminusBr2c <- var.Arc + var.Brc - 2 * cov.Ar1cBr2C
-
+  
   la.bot <- meanDiff - 2 * sqrt(var.Ar1cminusBr2c)
   la.top <- meanDiff + 2 * sqrt(var.Ar1cminusBr2c)
-
+  
   ci95meanDiff.bot <- meanDiff + qnorm(.025) * sqrt(var.MeanDiff)
   ci95meanDiff.top <- meanDiff + qnorm(.975) * sqrt(var.MeanDiff)
-
+  
   result2 <- data.frame(
     meanDiff = meanDiff, var.MeanDiff = var.MeanDiff, var.1obs = var.Ar1cminusBr2c,
     ci95meanDiff.bot = ci95meanDiff.bot, ci95meanDiff.top = ci95meanDiff.top,
     la.bot = la.bot, la.top = la.top, stringsAsFactors = TRUE )
-
-return(result2)
-
+  
+  return(result2)
+  
 }
 
 #---- getWRBM ----
-#' @title Get within-reader, between-modality paired data from an MRMC data file
+#' @title Get within-reader, between-modality paired data from an MRMC data frame
 #'
-#' @param mcsData A data frame with the following columns: readerID, caseID, modalityID, score
+#' @param mrmcDF A data frame with the following columns: readerID, caseID, modalityID, score
 #' @param modality.X The name of one modality
 #' @param modality.Y The name of one modality. This should be different from modality.X
 #'
-#' @return The result of merging the modality.X and modality.Y subsets of mcsData by readerID and caseID
+#' @return The result of merging the modality.X and modality.Y subsets of mrmcDF by readerID and caseID
 #' 
 #' @export
 #'
 # @examples
-getWRBM <- function(mcsData, modality.X, modality.Y) {
+getWRBM <- function(mrmcDF, modality.X, modality.Y) {
   
   # Make an "MRMClist" data frame for modality.X
-  df.x <- mcsData[mcsData$modalityID == modality.X,
+  df.x <- mrmcDF[mrmcDF$modalityID == modality.X,
                   c("readerID", "caseID", "modalityID", "score")]
   # Make an "MRMClist" data frame for modality.Y
-  df.y <- mcsData[mcsData$modalityID == modality.Y,
+  df.y <- mrmcDF[mrmcDF$modalityID == modality.Y,
                   c("readerID", "caseID", "modalityID", "score")]
   # Merge the two data frames
   df <- merge(df.x, df.y, by = c("readerID", "caseID"), suffixes = c(".X",".Y"))
@@ -190,7 +170,7 @@ getWRBM <- function(mcsData, modality.X, modality.Y) {
 }
 
 #---- getBRBM ----
-#' @title Get between-reader, between-modality paired data from an MRMC data file
+#' @title Get between-reader, between-modality paired data from an MRMC data frame
 #'
 #' @param mcsData A data frame with the following columns: readerID, caseID, modalityID, score
 #' @param modality.X The name of one modality
@@ -208,6 +188,11 @@ getBRBM <- function(mcsData, modality.X, modality.Y) {
   # This data frame will hold all the paired observations
   df <- data.frame()
   
+  # Make sure fields are factors
+  mcsData$readerID <- factor(mcsData$readerID)
+  mcsData$caseID <- factor(mcsData$caseID)
+  mcsData$modalityID <- factor(mcsData$modalityID)
+  
   # Split the data by readers
   nReaders <- nlevels(mcsData$readerID)
   mcsData <- split(mcsData, mcsData$readerID)
