@@ -1,60 +1,3 @@
-## convertDF ####
-#' Convert MRMC data frames
-#'
-#' @param inDF An MRMC dataframe with reading study results in \code{inDFtype} format
-#' @param inDFtype A string indicating the format type of the input MRMC data frame
-#' @param outDFtype A string indicating the format type of the output MRMC data frame. 
-#' @param readers A character array holding the column names (readerIDs) corresponding to the data from different readers
-#'
-#' @return An MRMC dataframe with reading study results in \code{outDFtype} format
-#' @export
-#' @details
-#' MRMC data frames contain scores from readers, cases, and sometimes modalities. This packages deals with (currently)
-#' two MRMC data frame formats. These are the formats:
-#' \describe{
-#'   \item{matrixMode}{
-#'     For this format, each row contains all the information for a case, 
-#'     including the reader study result for several readers, ground truth and other information as seperate columns.
-#'     This mode can only hold data from one modality.
-#'   }
-#'   \item{listMode}{
-#'     For this format, each row contains the data for one observation. The columns 
-#'     specify the readerID, caseID, score, ground truth, and other information if there is any.
-#'     This mode can hold data from multiple modalities.
-#'   }
-#' }
-#' 
-#'
-# @examples
-convertDF <- function(inDF, inDFtype, outDFtype, readers){
-  
-  if (inDFtype == "matrixMode" & outDFtype == "listMode"){
-    
-    # Split the data frame by columns, with and without the readers
-    dfReaders <- inDF[ , (colnames(inDF) %in% readers)]
-    dfNoReaders <- inDF[ , !(colnames(inDF) %in% readers)]
-    
-    # Replicate the data without readers while adding the data for each reader
-    outDF <- data.frame()
-    for (iReader in readers) {
-      
-      # Start with the data frame with no readers
-      tempDF <- dfNoReaders
-      # Add a column "readerID" and assign it the reader name
-      tempDF$readerID <- iReader
-      # Add a column "score" and assign it the scores of the current reader
-      tempDF$score <- dfReaders[ , iReader]
-      # Aggregate the reader data into the output data frame
-      outDF <- rbind(outDF, tempDF)
-      
-    }
-    
-    return(outDF)
-    
-  }
-  
-}
-
 ## createIMRMCdf ####
 #' Convert a data frame with all needed factors to doIMRMC formatted data frame
 #'
@@ -64,15 +7,16 @@ convertDF <- function(inDF, inDFtype, outDFtype, readers){
 #'        list(readerID = "***", caseID = "***", modalityID = "***", score = "***", truth="***")
 #' @param truePositiveFactor The true positive label, such as "cancer" or "1"
 #'
-#' @return output a doIMRMC formatted data frame: rows for truth and rows for data
+#' @return output a doIMRMC formatted data frame: rows for truth and rows for data.
+#' The results will be an iMRMC formatted data frame, see \link{dfMRMC_example}
 #'
 #' @export
 #'
 # @examples
 createIMRMCdf <- function(
-  dFrame,
-  keyColumns = list(readerID = "readerID", caseID = "caseID", modalityID = "modalityID", score = "score", truth = "truth"),
-  truePositiveFactor = "cancer"
+    dFrame,
+    keyColumns = list(readerID = "readerID", caseID = "caseID", modalityID = "modalityID", score = "score", truth = "truth"),
+    truePositiveFactor = "cancer"
 ){
 
   readerID <- keyColumns$readerID
@@ -138,7 +82,7 @@ createIMRMCdf <- function(
 #'
 # @examples
 undoIMRMCdf <- function(df.MRMC) {
-  
+
   # Separate the data frame into rows corresponding to truth
   # and rows corresping to observations
   df.Truth <- df.MRMC[df.MRMC$readerID == "-1", ]
@@ -174,7 +118,7 @@ undoIMRMCdf <- function(df.MRMC) {
 #' @export
 #'
 # @examples
-# # Simulate an MRMC data frame ####
+# # Simulate an MRMC data frame
 # simRoeMetz.config <- sim.gRoeMetz.config()
 # dFrameMRMC <- sim.gRoeMetz(simRoeMetz.config)
 # desc <- dFrameMRMC[
@@ -209,7 +153,7 @@ getMRMCscore <- function(df, iR, iC, modality) {
 #'   Leaving the levels (dropFlag = FALSE) is useful if you need the entire score
 #'   or design matrix when comparing or doing analyses with two modalities.
 #'
-#' @return A matrix [nCases, nReaders] of the scores each reader reported for each case
+#' @return A matrix that is \emph{nCases} by \emph{nReaders} of the scores each reader reported for each case
 #'
 #' @importFrom stats qnorm
 #'
@@ -242,7 +186,7 @@ convertDFtoScoreMatrix <- function(dfMRMC, modality = NULL, dropFlag = TRUE) {
   nCases <- nlevels(dfMRMC$caseID)
   nReaders <- nlevels(dfMRMC$readerID)
 
-  scores <- array(-1, c(nCases, nReaders), dimnames = list(caseIDs, readerIDs))
+  scores <- array(NA, c(nCases, nReaders), dimnames = list(caseIDs, readerIDs))
 
   index <- dfMRMC[ , c("caseID","readerID")]
   index <- data.matrix(index)
@@ -268,7 +212,7 @@ convertDFtoScoreMatrix <- function(dfMRMC, modality = NULL, dropFlag = TRUE) {
 #'   Leaving the levels (dropFlag = FALSE) is useful if you need the entire score
 #'   or design matrix when comparing or doing analyses with two modalities.
 #'
-#' @return A matrix [nCases, nReaders] indicating which scores were reported for each reader and case
+#' @return A matrix that is \emph{nCases} by \emph{nReaders} indicating which scores were reported for each reader and case
 #'
 #' @export
 #'
@@ -281,7 +225,7 @@ convertDFtoDesignMatrix <- function(dfMRMC, modality = NULL, dropFlag = TRUE) {
 
   # Reduce the modality factor to what is actually contained
   dfMRMC$modalityID <- factor(dfMRMC$modalityID)
-  
+
   # Check that there is data from one modality only.
   if (nlevels(dfMRMC$modalityID) != 1) {
     desc <- paste("This function only treats data sets with one modality.\n",
@@ -333,12 +277,12 @@ convertDFtoDesignMatrix <- function(dfMRMC, modality = NULL, dropFlag = TRUE) {
 #'
 # @examples
 extractPairedComparisonsBRBM <- function(
-  data0,
-  modalities = c("testA", "testB"),
-  keyColumns = list(readerID = "readerID",
-                    caseID = "caseID",
-                    modalityID = "modalityID",
-                    score = "score")
+    data0,
+    modalities = c("testA", "testB"),
+    keyColumns = list(readerID = "readerID",
+                      caseID = "caseID",
+                      modalityID = "modalityID",
+                      score = "score")
 ) {
 
   # Establish the key column names
@@ -404,19 +348,19 @@ extractPairedComparisonsBRBM <- function(
 #'
 #' @return A data frame of all paired observations.
 #'   Each observation comes from a one reader evaluating a case in two modalities
-#'   The first column corresponds to one reader evaluating the case in testA.
-#'   The second column corresonds to the same reader evaluating the case in testB.
+#'   The first column corresponds to one reader evaluating the case in "testA".
+#'   The second column corresonds to the same reader evaluating the case in "testB".
 #'
 #' @export
 #'
 # @examples
 extractPairedComparisonsWRBM <- function(
-  data0,
-  modalities = "testA",
-  keyColumns = list(readerID = "readerID",
-                    caseID = "caseID",
-                    modalityID = "modalityID",
-                    score = "score")
+    data0,
+    modalities = "testA",
+    keyColumns = list(readerID = "readerID",
+                      caseID = "caseID",
+                      modalityID = "modalityID",
+                      score = "score")
 ) {
 
   readerID <- keyColumns$readerID
@@ -533,33 +477,84 @@ renameCol <- function(df, oldColName, newColName) {
 #' @return a list of two data frames (df.tpf and df.fpf) both formatted for doIMRMC
 #' @export
 #'
-# @examples
+#' @examples
+#' # Create a sample configuration file
+#' config <- sim.gRoeMetz.config()
+#' # Simulate an MRMC ROC data set
+#' dFrame.imrmc <- sim.gRoeMetz(config)
+#' # Convert ROC MRMC data to TPF and FPF data frames
+#' result <- roc2binary(dFrame.imrmc, threshold = 0.9)
+#' # Analyze TPF data using doIMRMC
+#' tpf_result <- doIMRMC(result$df.tpf)
+#' # View(tpf_result$perReader)
+#
 roc2binary <- function(df.auc, threshold) {
+
+  # Separate truth and reader scores
   df.truth <- df.auc[df.auc$readerID == -1 | df.auc$readerID == "truth", ]
   df.scores <- df.auc[df.auc$readerID != -1 & df.auc$readerID != "truth", ]
 
+  # Separate truth negative and truth positive cases
   truth.neg <- droplevels(df.truth[df.truth$score == 0, ])
-  cases.neg <- levels(truth.neg$caseID)
-  nC.neg <- nlevels(truth.neg$caseID)
+  cases.neg <- unique(truth.neg$caseID)
+
+  # if(TRUE) print("check that number of unique cases = number of truth cases.
+  #              Insert MRMC format data check.")
+  ####
+  #### Check that each reader reads each case only once per modality
+  #### This also checks that each case only has one truth
+  #### If there are duplicates, throw an error
+  ####
+  index <- duplicated(df.auc[, c("readerID", "caseID", "modalityID")])
+  if (any(index)) {
+    desc <- paste(
+      "\n Some readers have read a case more than once in some modality.",
+      "\n Check these rows:",
+      "\n", paste((1:nrow(df.auc))[index], collapse = ", ")
+    )
+    stop(desc)
+  }
+
+  ####
+  #### Check that the number of unique cases = number of truth cases
+  ####
+  if(nrow(df.truth) != length(unique(df.scores$caseID))) {
+    desc <- paste(
+      "\n There is an uneven number of truth cases and unique reader cases.")
+    stop(desc)
+  }
 
   truth.pos <- droplevels(df.truth[df.truth$score == 1, ])
-  cases.pos <- levels(truth.pos$caseID)
-  nC.pos <- nlevels(truth.pos$caseID)
+  cases.pos <- unique(truth.pos$caseID)
 
+  # Determine if score is above/equiv./below given threshold
   df.scores$pos <- 0
+  df.scores$pos[df.scores$score == threshold] <- 0.5
   df.scores$pos[df.scores$score > threshold] <- 1
 
+  # Index positive truth cases and negative truth cases in df.scores (readers)
   index.pos <- as.character(df.scores$caseID) %in% cases.pos
   index.neg <- as.character(df.scores$caseID) %in% cases.neg
 
-  df.tpf <- df.scores[ , c("readerID", "caseID", "modalityID", "score")]
-  df.tpf$score <- 0.5
-  df.tpf$score[index.pos] <- df.scores$pos[index.pos]
+  # Calculate TPF dataframe
+  # Use positive truth index and pos (threshold adjusted) column
+  # Negative truth rows receive score of 0.5
+  df.tpf <- df.scores[ , c("readerID", "caseID", "modalityID", "score", "pos")]
+  df.tpf$success <- df.tpf$pos
+  df.tpf$score[index.neg] <- 0.5
+  df.tpf$score[index.pos] <- df.tpf$pos[index.pos]
+  df.tpf <- df.tpf[, c("readerID", "caseID", "modalityID", "score")]
 
-  df.fpf <- df.scores[ , c("readerID", "caseID", "modalityID", "score")]
-  df.fpf$score <- 0.5
-  df.fpf$score[index.neg] <- df.scores$pos[index.neg]
+  # Calculate FPF dataframe
+  # Use negative truth index and pos (threshold adjusted) column
+  # Positive truth rows receive score of 0.5
+  df.fpf <- df.scores[ , c("readerID", "caseID", "modalityID", "score", "pos")]
+  df.fpf$success <- df.fpf$pos
+  df.fpf$score[index.pos] <- 0.5
+  df.fpf$score[index.neg] <- df.fpf$pos[index.neg]
+  df.fpf <- df.fpf[, c("readerID", "caseID", "modalityID", "score")]
 
+  # Build dataframes of truth and TPF or FPF
   return(
     list(
       df.tpf = rbind(df.truth, df.tpf),
@@ -646,3 +641,177 @@ createDFdocumentation <- function(df) {
 
 }
 
+
+## deleteCol ####
+#' Delete a data frame column
+#'
+#' @param df A data frame
+#'
+#' @param colName Column name or list of column names to be deleted
+#'
+#' @return The data frame without the deleted column or columns
+#'
+#' @export
+#'
+# @examples
+deleteCol <- function(df, colName) {
+  df <- df[ , -which(names(df) %in% c(colName))]
+  return(df)
+}
+
+
+## getMRMCdataset ####
+#' Import MRMC dataset from the web (https://github.com/DIDSR/iMRMC/wiki/iMRMC-Datasets)
+#'
+#' @param dataset Possible dataset options available:
+#' \itemize{
+#'   \item \strong{pilotHTT}: Breast cancer annotation data that is the aggregate of all clean data from the HTT project pilot study. https://github.com/DIDSR/HTT
+#'   \item \strong{viperObs}: Individual observations of each reader reading each case from the FDA Validation of Imaging Premarket Evaluation and Regulation (VIPER) Study. https://github.com/DIDSR/viperData
+#'   \item \strong{viperObs365}: Individual observations of each reader reading each case from the FDA Validation of Imaging Premarket Evaluation and Regulation (VIPER) Study, truth is based on cancer status at 365 days. https://github.com/DIDSR/viperData
+#'   \item \strong{viperObs455}: Individual observations of each reader reading each case from the FDA Validation of Imaging Premarket Evaluation and Regulation (VIPER) Study, truth is based on cancer status at 455 days. https://github.com/DIDSR/viperData
+#'   \item \strong{MFcounts_dfClassify}: A data frame comparing mitotic figure counting performance based on whole slide images (WSI images) from four scanners to the counts from a microscope, data is per  candidate mitotic figure and modality. https://github.com/DIDSR/mitoticFigureCounts/tree/master
+#'   \item \strong{MFcounts_dfCountROI}: A data frame comparing mitotic figure counting performance based on whole slide images (WSI images) from four scanners to the counts from a microscope, data is per ROI and modality. https://github.com/DIDSR/mitoticFigureCounts/tree/master
+#'   \item \strong{MFcounts_dfCountWSI}: A data frame comparing mitotic figure counting performance based on whole slide images (WSI images) from four scanners to the counts from a microscope, data is per WSI and modality. https://github.com/DIDSR/mitoticFigureCounts/tree/master
+#'   \item \strong{cardioStudyTruth}: Cardio CT data comparing display color scale effects on diagnostic performance and reader agreement, ground truth data. https://github.com/DIDSR/colorScaleStudyData
+#'   \item \strong{cardioStudyRawData}: Cardio CT data comparing display color scale effects on diagnostic performance and reader agreement, reader data. https://github.com/DIDSR/colorScaleStudyData
+#'   \item \strong{prostateTruth}: Prostate MRI data comparing display color scale effects on diagnostic performance and reader agreement, ground truth data. https://github.com/DIDSR/colorScaleStudyData
+#'   \item \strong{prostateRawData}: Prostate MRI data comparing display color scale effects on diagnostic performance and reader agreement, reader data. https://github.com/DIDSR/colorScaleStudyData
+#' }
+#'
+#' @return desired dataset downloaded from the web as a csv
+#'
+#' @import utils
+#' @export
+#'
+#' @examples
+#' # Save Prostate MRI ground truth and reader data
+#' truthData <- getMRMCdataset("prostateTruth")
+#' rawData <- getMRMCdataset("prostateRawData")
+#
+getMRMCdataset <- function(dataset = c("pilotHTT", "viperObs", "viperObs365",
+                                       "viperObs455", "MFcounts_dfClassify",
+                                       "MFcounts_dfCountROI", "MFcounts_dfCountWSI",
+                                       "cardioStudyTruth", "cardioStudyRawData",
+                                       "prostateTruth", "prostateRawData")){
+
+  if(dataset == "pilotHTT") {
+    link <- "https://raw.githubusercontent.com/DIDSR/HTT/main/inst/extdata/pilotHTT.csv"
+    pilotHTT <- read.csv(link)
+    print("Annotation data that is the aggregate of all clean data from the HTT project pilot study.")
+    print("Repository where you can find more information about the data: https://github.com/DIDSR/HTT")
+
+    return(pilotHTT)
+  }
+
+  if(dataset == "viperObs") {
+    link <- "https://raw.githubusercontent.com/DIDSR/viperData/master/inst/data-raw/viperObs.csv"
+    viperObs <- read.csv(link)
+    print("Individual observations of each reader reading each case. Simplified and merged version of data based on `viperObs365` and `viperObs455`.")
+    print("Repository where you can find more information about the data: https://github.com/DIDSR/viperData")
+
+    return(viperObs)
+  }
+
+  if(dataset == "viperObs365") {
+    link <- "https://raw.githubusercontent.com/DIDSR/viperData/master/inst/data-raw/viperObs365.csv"
+    viperObs365 <- read.csv(link)
+    viperObs365 <- viperObs365[, -1]
+    print("Individual observations of each reader reading each case. Truth labels are based on cancer at 365 days.")
+    print("Repository where you can find more information about the data: https://github.com/DIDSR/viperData")
+
+    return(viperObs365)
+  }
+
+  if(dataset == "viperObs455") {
+    link <- "https://raw.githubusercontent.com/DIDSR/viperData/master/inst/data-raw/viperObs455.csv"
+    viperObs455 <- read.csv(link)
+    viperObs455 <- viperObs455[, -1]
+    print("Individual observations of each reader reading each case. Truth labels are based on cancer at 455 days.")
+    print("Repository where you can find more information about the data: https://github.com/DIDSR/viperData")
+
+    return(viperObs455)
+  }
+
+  if(dataset == "MFcounts_dfClassify") {
+    link <- "https://raw.githubusercontent.com/DIDSR/mitoticFigureCounts/master/data/dfClassify20180627.csv"
+    MFcounts_dfClassify <- read.csv(link)
+    print("A single data frame of the study data. Each row corresponds to a candidate mitotic figure and modality (155 candidates x 5 modalities = 775 rows). There is a column for each observer and the truth.")
+    print("Repository where you can find more information about the data: https://github.com/DIDSR/mitoticFigureCounts/tree/master")
+
+    return(MFcounts_dfClassify)
+  }
+
+  if(dataset == "MFcounts_dfCountROI") {
+    link <- "https://raw.githubusercontent.com/DIDSR/mitoticFigureCounts/master/data/dfCountROI20180627.csv"
+    MFcounts_dfCountROI <- read.csv(link)
+    print("A data frame of the mitotic figure counts per ROI and modality (40 ROIs x 5 modalities = 200 rows). There is a column for each observer and the truth.")
+    print("Repository where you can find more information about the data: https://github.com/DIDSR/mitoticFigureCounts/tree/master")
+
+    return(MFcounts_dfCountROI)
+  }
+
+  if(dataset == "MFcounts_dfCountWSI") {
+    link <- "https://raw.githubusercontent.com/DIDSR/mitoticFigureCounts/master/data/dfCountWSI20180627.csv"
+    MFcounts_dfCountWSI <- read.csv(link)
+    print("A single data frame of the mitotic figure counts per WSI and modality (4 WSIs x 5 modalities = 20 rows). There is a column for each observer and the truth. ")
+    print("Repository where you can find more information about the data: https://github.com/DIDSR/mitoticFigureCounts/tree/master")
+
+    return(MFcounts_dfCountWSI)
+  }
+
+  if(dataset == "cardioStudyTruth") {
+    link <- "https://raw.githubusercontent.com/DIDSR/colorScaleStudyData/master/data-raw/Cardio%20Study_truth.csv"
+    cardioStudyTruth <- read.csv(link)
+    print("Truth of each case in study. From left to right each column corresponds to case number and truth. 1 is assigned to cases with lesion (positive cases) and 0 to cases without lesion (negative cases).")
+    print("You may also want to get the `cardioStudyRawData` dataset.")
+    print("Repository where you can find more information about the data: https://github.com/DIDSR/colorScaleStudyData")
+
+    names(cardioStudyTruth) <- c("caseID", "score")
+    cardioStudyTruth$readerID <- "truth"
+    cardioStudyTruth$modalityID <- "truth"
+
+    return(cardioStudyTruth)
+  }
+
+  if(dataset == "cardioStudyRawData") {
+    link <- "https://raw.githubusercontent.com/DIDSR/colorScaleStudyData/master/data-raw/CardioCTstudy_3mod_12obs_210cases_rawdata.csv"
+    cardioStudyRawData <- read.csv(link)
+    print("Observations of each case in study. From left to right each column corresponds to reader, case number, modality (color scale), confidence score assigned. Grayscale was evaluated using GSDF settings and Rainbow and Hotiron using RGB settings.")
+    print("You may also want to get the `cardioStudyTruth` dataset.")
+    print("Repository where you can find more information about the data: https://github.com/DIDSR/colorScaleStudyData")
+
+    names(cardioStudyRawData) <- c("readerID", "caseID", "modalityID", "score")
+
+    return(cardioStudyRawData)
+  }
+
+  if(dataset == "prostateTruth") {
+    link <- "https://raw.githubusercontent.com/DIDSR/colorScaleStudyData/master/data-raw/Prostate_Truth.csv"
+    prostateTruth <- read.csv(link)
+    print("Truth of each case in study. From left to right each column corresponds to case number and truth. 1 is assigned to cases with lesion (positive cases) and 0 to cases without lesion (negative cases).")
+    print("You may also want to get the `prostateRawData` dataset.")
+    print("Repository where you can find more information about the data: https://github.com/DIDSR/colorScaleStudyData")
+
+    names(prostateTruth) <- c("caseID", "score")
+    prostateTruth$readerID <- "truth"
+    prostateTruth$modalityID <- "truth"
+
+    return(prostateTruth)
+  }
+
+  if(dataset == "prostateRawData") {
+    link <- "https://raw.githubusercontent.com/DIDSR/colorScaleStudyData/master/data-raw/Prostate_5mod_9obs_165cases_wGSDFrawdata_Allreaders.csv"
+    prostateRawData <- read.csv(link)
+    print("Observations of each case in the study. From left to right each column corresponds to reader, case number, modality (color scale), confidence score assigned. Unless otherwise stated in the modality name Grayscale was evaluated using GSDF settings and Rainbow and Hotiron using RGB settings.")
+    print("You may also want to get the `prostateTruth` dataset.")
+    print("Repository where you can find more information about the data: https://github.com/DIDSR/colorScaleStudyData")
+
+    names(prostateRawData) <- c("readerID", "caseID", "modalityID", "score")
+    return(prostateRawData)
+  }
+
+    print("The given data name is not an MRMC data object from
+          https://github.com/DIDSR/iMRMC/wiki/iMRMC-Datasets.")
+    print("Please input one of the specified data names.")
+
+}
